/*
 *   copyright            : (C) 2001-2002 by Richard Moore
 *   email                : rich@kde.org
 */

#include <qapplication.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtimer.h>
#include <qvbox.h>

#include <kdialog.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <kglobalsettings.h>
#include <netwm.h>

#include "kpassivepopup.h"
#include "kpassivepopup.moc"

static const int DEFAULT_POPUP_TIME = 6*1000;
static const int POPUP_FLAGS = Qt::WStyle_Customize | Qt::WDestructiveClose | Qt::WX11BypassWM
                             | Qt::WStyle_StaysOnTop | Qt::WStyle_Tool | Qt::WStyle_NoBorder;

KPassivePopup::KPassivePopup( QWidget *parent, const char *name )
    : QFrame( 0, name, POPUP_FLAGS ),
      owner( parent ), window( parent->winId() ), msgView( 0 ), layout( 0 ),
      hideDelay( DEFAULT_POPUP_TIME ), hideTimer( 0 ), d( 0 )
{
    setFrameStyle( QFrame::Box| QFrame::Plain );
    setLineWidth( 2 );
}

KPassivePopup::KPassivePopup( WId win, const char *name )
    : QFrame( 0, name, POPUP_FLAGS ),
      owner( 0 ), window( win ), msgView( 0 ), layout( 0 ),
      hideDelay( DEFAULT_POPUP_TIME ), hideTimer( 0 ), d( 0 )
{
    setFrameStyle( QFrame::Box| QFrame::Plain );
    setLineWidth( 2 );
}

KPassivePopup::~KPassivePopup()
{
}

void KPassivePopup::setView( QWidget *child )
{
    msgView = child;
    delete layout;
    layout = new QVBoxLayout( this, KDialog::marginHint(), KDialog::spacingHint() );
    layout->addWidget( msgView );
    layout->activate();
}

void KPassivePopup::setView( const QString &caption, const QString &text, const QPixmap &icon )
{
    QVBox *vb = new QVBox( this );
    QWidget *w = vb;
    if ( !icon.isNull() ) {
	QHBox *hb = new QHBox( vb );
	hb->setMargin( KDialog::marginHint() );
	hb->setSpacing( KDialog::spacingHint() );
	w = hb;
	ttlIcon = new QLabel( hb, "title_icon" );
	ttlIcon->setPixmap( icon );
    }

    if ( !caption.isNull() ) {
	ttl = new QLabel( caption, w, "title_label" );
	QFont fnt = ttl->font();
	fnt.setBold( true );
	ttl->setFont( fnt );
    };

    msg = new QLabel( text, vb, "msg_label" );

    setView( vb );
}

void KPassivePopup::setView( const QString &text, const QString &caption )
{
    setView( text, caption, QPixmap() );
}

void KPassivePopup::setTimeout( int delay )
{
    hideDelay = delay;
    if ( hideTimer )
	hideTimer->changeInterval( delay );
}

void KPassivePopup::mouseReleaseEvent( QMouseEvent *e )
{
    emit clicked();
    emit clicked( e->pos() );
}

//
// Main Implementation
//

void KPassivePopup::show()
{
    if ( size() != sizeHint() )
	resize( sizeHint() );   

    positionSelf();
    QFrame::show();

    if ( hideDelay >= 0 ) {
	hideTimer = new QTimer( this, "hide_timer" );
	connect( hideTimer, SIGNAL( timeout() ), SLOT( hide() ) );
	hideTimer->start( hideDelay );
    }
}

void KPassivePopup::positionSelf()
{
    NETWinInfo ni( qt_xdisplay(),
		   (owner ? owner->winId() : window),
		   qt_xrootwin(), 
		   NET::WMIconGeometry | NET::WMKDESystemTrayWinFor );

    // Figure out where to put the popup. Note that we must handle
    // windows that skip the taskbar cleanly
    QRect target;
    if ( ni.kdeSystemTrayWinFor() ) {
	NETRect frame, win;
	ni.kdeGeometry( frame, win );
	target.setRect( win.pos.x, win.pos.y, win.size.width, win.size.height );
    }
    else if ( ni.state() & NET::SkipTaskbar ) {
	target.setRect( 0, 0, 0, 0 );
    }
    else {
	NETRect r = ni.iconGeometry();
	target.setRect( r.pos.x, r.pos.y, r.size.width, r.size.height );
    }

    moveNear( target );
}

void KPassivePopup::moveNear( QRect target )
{
    QPoint pos = target.topLeft();
    int x = pos.x();
    int y = pos.y();
    int w = width();
    int h = height();

    if ( x < ( qApp->desktop()->width() / 2 ) )
	x = x + target.width();
    else
	x = x - w;

    if ( (y + h) > qApp->desktop()->height() )
	y = y - h + target.height();
    
#ifdef OLD_BITS
    if ( (x - w) >= 0  )
	x = x - w;
#endif

    move( x, y );
}

//
// Convenience Methods
//

KPassivePopup *KPassivePopup::message( const QString &text,
				       const QString &caption, const QPixmap &icon,
				       QWidget *parent, const char *name )
{
    KPassivePopup *pop = new KPassivePopup( parent, name );
    pop->setView( text, caption, icon );
    pop->show();

    return pop;
}

KPassivePopup *KPassivePopup::message( const QString &text, QWidget *parent, const char *name )
{
    return message( text, QString::null, QPixmap(), parent, name );
}

KPassivePopup *KPassivePopup::message( const QString &text, const QString &caption,
				       QWidget *parent, const char *name )
{
    return message( text, caption, QPixmap(), parent, name );
}

// Local Variables:
// c-basic-offset: 4
// End:


